import time
import pytest

from django.core.exceptions import SuspiciousOperation

from greg.models import Identity
from gregui.authentication.auth_backends import GregOIDCBackend
from gregui.models import GregUserProfile


pytestmark = pytest.mark.django_db


def test_validate_issuer(id_token_payload):
    backend = GregOIDCBackend()
    backend.validate_issuer(id_token_payload)
    id_token_payload["iss"] = "http://suspicious.no"
    with pytest.raises(SuspiciousOperation):
        backend.validate_issuer(id_token_payload)


def test_validate_audiences(id_token_payload):
    backend = GregOIDCBackend()
    backend.validate_audience(id_token_payload)

    id_token_payload["aud"] = [id_token_payload["aud"], "other_aud"]
    with pytest.raises(SuspiciousOperation):
        backend.validate_audience(id_token_payload)


def test_validate_expiry(id_token_payload):
    backend = GregOIDCBackend()
    with pytest.raises(SuspiciousOperation):
        backend.validate_expiry(id_token_payload)

    id_token_payload["exp"] = int(time.time()) + 3600
    backend.validate_expiry(id_token_payload)


def test_filter_users(greg_users, claims):
    backend = GregOIDCBackend()
    user = backend.filter_users_by_claims(claims).get()
    assert user.username == greg_users["https://auth.dataporten.nosubsub"].username

    claims["sub"] = "non-existant-sub"
    users = backend.filter_users_by_claims(claims)
    assert len(users) == 0


def test_create_user(claims):
    backend = GregOIDCBackend()
    user = backend.create_user(claims)
    assert user.first_name == "Frank Foreleser"
    assert user.last_name == "Føllesen"
    assert user.email == "noreply@feide.no"

    userProfile = GregUserProfile.objects.get(user=user)
    assert userProfile

    person = userProfile.person
    assert person.first_name == user.first_name
    assert person.last_name == user.last_name
    assert person.email == user.email

    ids = Identity.objects.get(person=person, type="feide_id")
    assert ids.value == "frank_foreleser@spusers.feide.no"


def test_update_user(greg_users, claims):
    backend = GregOIDCBackend()
    user = backend.update_user(None, claims)

    assert user.first_name == "Frank Foreleser"
    assert user.last_name == "Føllesen"
    assert user.email == "noreply@feide.no"
